import 'dart:async';
import 'dart:io';

import "package:http/http.dart" as Http;
import 'package:path_provider/path_provider.dart';
import 'package:speedq_net/entities/TokenResponse.dart';

import 'package:speedq_net/src/util/json.dart';

/// http client to send api calls
class HttpClient {
    static final String _root = "https://skipdq.herokuapp.com";
    static final _Client _client = new _Client();

    static void authorise(String token) {
        _client.token = token;
    }

    static void refresh(String id, String refresh, Function callback) {
        post("/refresh", "{\"id\": \"$id\", \"refresh\": \"$refresh\"}", (Http.Response response) {
            // get token response
            TokenResponse tokenResponse = TokenResponse.fromJson(response.body);
            authorise(tokenResponse.access);
            // write to file
            // get documents dir
            getApplicationDocumentsDirectory().then((dir) {
                // write to file
                File("${dir.path}/token.json").writeAsString(response.body);
            });
            callback();
        });
    }

    static void get<T>(String endPoint, {void Function(T)? callback, void Function(List<T>)? listCallback}) {
        // callback cuz dart is gay
        _client.read(Uri.parse("$_root$endPoint")).then((json) => listCallback != null ? listCallback(createFromJsonArray(json)) : callback!(create(json)));
    }

    static void getResponse(String endPoint, void Function(Http.Response) callback) {
        _client.get(Uri.parse("$_root$endPoint")).then((response) => callback(response));
    }

    static void post(String endPoint, String json, [void Function(Http.Response)? callback]) {
        if(callback == null) {
            _client.post(Uri.parse("$_root$endPoint"), body: json);
        } else {
            _client.post(Uri.parse("$_root$endPoint"), body: json).then((response) => callback(response));
        }
    }

    static void put(String endPoint, String json) {
        _client.put(Uri.parse("$_root$endPoint"), body: json);
    }

    static void patch(String endPoint, String json) {
        _client.patch(Uri.parse("$_root$endPoint"), body: json);
    }

    static void delete(String endPoint, [String? json]) {
        _client.delete(Uri.parse("$_root$endPoint"), body: json);
    }

    static String getRoot() {
        return _root;
    }
}

class _Client extends Http.BaseClient {
    String token = "";
    final Http.Client _inner = new Http.Client();

    @override
    Future<Http.StreamedResponse> send(Http.BaseRequest request) {
        // add auth token to all requests
        request.headers["Authorization"] = token.isNotEmpty ? "Bearer $token" : "";
        request.headers["content-type"] = "application/json";
        return _inner.send(request);
    }

}